/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef A11YPREFERENCESDIALOG_H
#define A11YPREFERENCESDIALOG_H

#include <QWidget>
#include "clib-syslog.h"


namespace Ui {
class A11yPreferencesDialog;
}

class A11yPreferencesDialog : public QWidget
{
    Q_OBJECT

public:
    explicit A11yPreferencesDialog(QWidget *parent = nullptr);
    ~A11yPreferencesDialog();

Q_SIGNALS:

    void singalCloseWidget();

protected:
    void closeEvent(QCloseEvent *event);

private:
    Ui::A11yPreferencesDialog *ui;



};

#endif // A11YPREFERENCESDIALOG_H
